/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package storage.database;

import domain.Manufacturer;
import domain.Product;
import domain.User;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.List;

/**
 *
 * @author student1
 */
public class DatabaseBroker {

    private Connection connection;
    private static DatabaseBroker instance;

    private DatabaseBroker() throws SQLException {
        String url = "jdbc:mysql://localhost/soft_dev_d4";
        String user = "root";
        String password = "";
        connection = DriverManager.getConnection(url, user, password);
        connection.setAutoCommit(false);
        // Zahteva eksplicitnu potvrdu transakcije
        System.out.println("Connection OK");
    }

    public void commitTransaction() throws SQLException {
        connection.commit();
    }

    public void rollbackTransaction() throws SQLException {
        connection.rollback();
    }

    public static DatabaseBroker getInstance() throws SQLException {
        if (instance == null) {
            instance = new DatabaseBroker();
        }
        return instance;
    }

    public List<Manufacturer> getAllManufacturers() throws Exception {
        List<Manufacturer> manufacturers = new LinkedList<>();
        String query = "SELECT * FROM manufacturer";
        try {
            Statement statement = connection.createStatement();
            ResultSet resultSet = statement.executeQuery(query);
            while (resultSet.next()) {
                Long id = resultSet.getLong("id");
                String name = resultSet.getString(2);
                Manufacturer manufacturer = new Manufacturer(id, name);
                manufacturers.add(manufacturer);
            }
            resultSet.close();
            statement.close();
        } catch (SQLException ex) {
            System.out.println(ex.getMessage());
            throw new Exception(ex.getMessage(), ex);
        }
        return manufacturers;
    }

    public User login(String username, String password) throws Exception {
        //String query = "SELECT * FROM User WHERE username = '" + username + "' AND password = '" + password + "'";
        String query = "SELECT * FROM User WHERE username = ? AND password = ?";
        try {
            PreparedStatement preparedStatement = connection.prepareStatement(query);
            preparedStatement.setString(1, username);
            preparedStatement.setString(2, password);
            ResultSet resultSet = preparedStatement.executeQuery();
            if (resultSet.next()) {
                Long id = resultSet.getLong("id");
                String firstName = resultSet.getString("firstName");
                String lastName = resultSet.getString("lastName");
                String email = resultSet.getString("email");
                User user = new User(id, username, password, firstName, lastName, email);
                resultSet.close();
                preparedStatement.close();
                return user;
            }
        } catch (SQLException ex) {
            System.out.println(ex.getMessage());
        }
        throw new Exception("Ivalid username and password!");
    }

    public void saveProduct(Product product) throws Exception {
        try {
            String query = "INSERT INTO Product(name, description, measurementUnit, price, manufacturer_id, user_id) VALUES (?,?,?,?,?,?)";
            PreparedStatement preparedStatement = connection.prepareStatement(query);
            preparedStatement.setString(1, product.getName());
            preparedStatement.setString(2, product.getDescription());
            preparedStatement.setString(3, product.getMeasurementUnit().toString());
            preparedStatement.setBigDecimal(4, product.getPrice());
            preparedStatement.setLong(5, product.getManufacturer().getId());
            preparedStatement.setLong(6, product.getSavedUser().getId());
            preparedStatement.executeUpdate();
            preparedStatement.close();
        } catch (SQLException ex) {
            throw new Exception("Save product exception! " + ex.getMessage(), ex);
        }
    }

    public void saveProducts(List<Product> products) throws Exception {
        try {
            String query = "INSERT INTO Product(name, description, measurementUnit, price, manufacturer_id, user_id) VALUES (?,?,?,?,?,?)";
            PreparedStatement preparedStatement = connection.prepareStatement(query);
            for (Product product : products) {
                preparedStatement.setString(1, product.getName());
                preparedStatement.setString(2, product.getDescription());
                preparedStatement.setString(3, product.getMeasurementUnit().toString());
                preparedStatement.setBigDecimal(4, product.getPrice());
                preparedStatement.setLong(5, product.getManufacturer().getId());
                preparedStatement.setLong(6, product.getSavedUser().getId());
                preparedStatement.executeUpdate();
            }
            preparedStatement.close();
            commitTransaction();
        } catch (SQLException ex) {
            if (connection != null) {
                rollbackTransaction();
            }
            ex.printStackTrace();
            throw new Exception("Save product exception! " + ex.getMessage(), ex);
        }

    }

}
