/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package storage.database;

import domain.Manufacturer;
import domain.MeasurementUnit;
import domain.Product;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import storage.AbstractStorageProduct;
import storage.database.connection.DatabaseConnection;

/**
 *
 * @author student1
 */
public class DatabaseStorageProduct  extends AbstractStorageProduct {

    private final DatabaseStorageUser databaseStorageUser;

    public DatabaseStorageProduct() {
        databaseStorageUser = new DatabaseStorageUser();
    }

    @Override
    public Product findById(Long entityId) throws Exception {
        try {
            Product product = null;

            Connection connection = DatabaseConnection.getInstance().getConnection();
            String query = "select p.id, p.name, p.description, p.measurementUnit, p.price, m.id as manufacturerId, m.name as manufacturer, u.id as userId "
                    + "from product p "
                    + "inner join manufacturer m on p.manufacturer_id = m.id "
                    + "inner join user u on p.user_id = u.id where p.id=" + entityId;
            System.out.println(query);
            Statement statement = connection.createStatement();
            ResultSet resultSet = statement.executeQuery(query);

            while (resultSet.next()) {
                Long id = resultSet.getLong("id");
                String productName = resultSet.getString("name");
                String description = resultSet.getString("description");
                MeasurementUnit measurementUnit = MeasurementUnit.valueOf(resultSet.getString("measurementUnit"));
                BigDecimal price = resultSet.getBigDecimal("price");

                Long manufacturerId = resultSet.getLong("manufacturerId");
                String manufacturerName = resultSet.getString("manufacturer");
                Manufacturer manufacturer = new Manufacturer(manufacturerId, manufacturerName);

                Long userId = resultSet.getLong("userId");

                product = new Product();
                product.setId(id);
                product.setName(productName);
                product.setDescription(description);
                product.setMeasurementUnit(measurementUnit);
                product.setPrice(price);
                product.setSavedUser(databaseStorageUser.findById(userId));
                product.setManufacturer(manufacturer);
            }
            resultSet.close();
            statement.close();

            if (product == null) {
                throw new Exception();
            }
            return product;
        } catch (SQLException ex) {
            System.out.println(ex.getMessage());
        }
        throw new Exception("Product does not exist");
    }

    @Override
    public List<Product> getAll() throws Exception {
        List<Product> products = new ArrayList<>();
        try {
            Connection connection = DatabaseConnection.getInstance().getConnection();
            String query = "select p.id, p.name, p.description, p.measurementUnit, p.price, m.id as manufacturerId, m.name as manufacturer, u.id as userId "
                    + "from product p "
                    + "inner join manufacturer m on p.manufacturer_id = m.id "
                    + "inner join user u on p.user_id = u.id";
            System.out.println(query);
            Statement statement = connection.createStatement();
            ResultSet resultSet = statement.executeQuery(query);

            while (resultSet.next()) {
                Long id = resultSet.getLong("id");
                String productName = resultSet.getString("name");
                String description = resultSet.getString("description");
                MeasurementUnit measurementUnit = MeasurementUnit.valueOf(resultSet.getString("measurementUnit"));
                BigDecimal price = resultSet.getBigDecimal("price");

                Long manufacturerId = resultSet.getLong("manufacturerId");
                String manufacturerName = resultSet.getString("manufacturer");
                Manufacturer manufacturer = new Manufacturer(manufacturerId, manufacturerName);

                Long userId = resultSet.getLong("userId");

                Product product = new Product();
                product.setId(id);
                product.setName(productName);
                product.setDescription(description);
                product.setMeasurementUnit(measurementUnit);
                product.setPrice(price);
                product.setSavedUser(databaseStorageUser.findById(userId));
                product.setManufacturer(manufacturer);

                products.add(product);
            }
            resultSet.close();
            statement.close();

            return products;
        } catch (SQLException ex) {
            System.out.println(ex.getMessage());
        }
        return products;
    }

    @Override
    public void add(Product product) throws SQLException {
        Connection connection = null;
        try {
            connection = DatabaseConnection.getInstance().getConnection();
            String query = "insert into product(name, description,"
                    + " manufacturer_id, measurementUnit,user_id, price) values(?,?,?,?,?,?)";
            PreparedStatement preparedStatement = connection.prepareStatement(query);
            preparedStatement.setString(1, product.getName());
            preparedStatement.setString(2, product.getDescription());
            preparedStatement.setLong(3, product.getManufacturer().getId());
            preparedStatement.setString(4, product.getMeasurementUnit().toString());
            preparedStatement.setLong(5, product.getSavedUser().getId());
            preparedStatement.setBigDecimal(6, product.getPrice());

            preparedStatement.executeUpdate();
            preparedStatement.close();
            connection.commit();
        } catch (Exception e) {
            if (connection != null) {
                connection.rollback();
            }
            throw e;
        }
    }

    public void addAll(List<Product> products) throws Exception {
        Connection connection = null;
        try {
            connection = DatabaseConnection.getInstance().getConnection();
            String query = "insert into product(name, description,"
                    + " manufacturer_id, measurementUnit,user_id, price) values(?,?,?,?,?,?)";
            PreparedStatement preparedStatement = connection.prepareStatement(query);
            for (Product product : products) {
                preparedStatement.setString(1, product.getName());
                preparedStatement.setString(2, product.getDescription());
                preparedStatement.setLong(3, product.getManufacturer().getId());
                preparedStatement.setString(4, product.getMeasurementUnit().toString());
                preparedStatement.setLong(5, product.getSavedUser().getId());
                preparedStatement.setBigDecimal(6, product.getPrice());

                preparedStatement.executeUpdate();
            }

            preparedStatement.close();
            connection.commit();
        } catch (Exception e) {
            if (connection != null) {
                connection.rollback();
            }
            e.printStackTrace();
            throw e;
        }
    }
}
