/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package server;

import controller.Controller;
import domain.IGeneralEntity;
import domain.User;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import transfer.Request;
import transfer.Response;
import transfer.util.Operation;
import transfer.util.ResponseStatus;

/**
 *
 * @author student1
 */
public class ClientThread extends Thread {

    private Socket socket;

    public ClientThread(Socket socket) {
        this.socket = socket;
    }

    @Override
    public void run() {
        try {
            communication();
        } catch (Exception ex) {
            Logger.getLogger(ClientThread.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void communication() throws Exception {

        while (!isInterrupted()) {
            ObjectInputStream inSocket = new ObjectInputStream(socket.getInputStream());
            Object object = inSocket.readObject();
            if (object instanceof Request) {
                Request request = (Request) object;
                Response response = handleRequest(request);
                sendResponse(socket, response);
            }
        }
    }

    private Response handleRequest(Request request) {
        int operation = request.getOperation();
        Response response = new Response();

        switch (operation) {
            case Operation.OPERATION_LOGIN:
                User requestUser = (User) request.getData();
                 {
                    try {
                        User user = Controller
                                .getInstance()
                                .logIn(requestUser.getUsername(), requestUser.getPassword());
                        response.setStatus(ResponseStatus.SUCCESS);
                        response.setData(user);
                    } catch (Exception ex) {
                        Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
                        response.setStatus(ResponseStatus.ERROR);
                        response.setError(ex);
                    }
                }
                break;
            case Operation.OPERATION_GET_ALL_MANUFACTURERS: {
                try {
                    List<IGeneralEntity> manufacturers = Controller.getInstance().getAllManufacturers();
                    response.setStatus(ResponseStatus.SUCCESS);
                    response.setData(manufacturers);
                } catch (Exception ex) {
                    Logger.getLogger(ClientThread.class.getName()).log(Level.SEVERE, null, ex);
                    Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
                    response.setStatus(ResponseStatus.ERROR);
                    response.setError(ex);
                }
            }
            break;
        }

        return response;
    }

    private void sendResponse(Socket socket, Response response) throws IOException {
        ObjectOutputStream outSocket = new ObjectOutputStream(socket.getOutputStream());
        outSocket.writeObject(response);
    }

}
